@@ -1,4 +1,19 @@
-"""Numba-compiled 1-dim and 2-dim functions.
+"""Numba-compiled functions.
+
+Provides an arsenal of Numba-compiled functions that are used by accessors
+and in many other parts of the backtesting pipeline, such as technical indicators.
+These only accept NumPy arrays and other Numba-compatible types.
+
+The module can be accessed directly via `vbt.nb`.
+
+```python-repl
+>>> import numpy as np
+>>> import vectorbt as vbt
+
+>>> # vectorbt.generic.nb.rolling_mean_1d_nb
+>>> vbt.nb.rolling_mean_1d_nb(np.array([1, 2, 3, 4]), 2)
+array([nan, 1.5, 2.5, 3.5])
+```
 
 !!! note
     vectorbt treats matrices as first-class citizens and expects input arrays to be
@@ -39,24 +54,6 @@ def shuffle_nb(a, seed=None):
     return out
 
 
-@njit(cache=True)
-def prepend_1d_nb(a, n, value):
-    """Prepend value `n` times."""
-    out = np.empty(a.shape[0] + n, dtype=np.float_)
-    out[:n] = value
-    out[n:] = a
-    return out
-
-
-@njit(cache=True)
-def prepend_nb(a, n, value):
-    """2-dim version of `prepend_1d_nb`."""
-    out = np.empty((a.shape[0] + n, a.shape[1]), dtype=np.float_)
-    out[:n, :] = value
-    out[n:, :] = a
-    return out
-
-
 @njit(cache=True)
 def set_by_mask_1d_nb(a, mask, value):
     """Set each element to a value by boolean mask."""
@@ -107,6 +104,26 @@ def fillna_nb(a, value):
     return set_by_mask_nb(a, np.isnan(a), value)
 
 
+@njit(cache=True)
+def bshift_1d_nb(a, n):
+    """Shift backward by `n` positions.
+
+    Numba equivalent to `pd.Series(a).shift(value)`.
+
+    !!! warning
+        May introduce look-ahead bias."""
+    out = np.empty_like(a, dtype=np.float_)
+    out[-n:] = np.nan
+    out[:-n] = a[n:]
+    return out
+
+
+@njit(cache=True)
+def bshift_nb(a, n):
+    """2-dim version of `bshift_1d_nb`."""
+    return bshift_1d_nb(a, n)
+
+
 @njit(cache=True)
 def fshift_1d_nb(a, n):
     """Shift forward by `n` positions.
@@ -195,7 +212,7 @@ def product_1d_nb(a):
     Numba equivalent to `pd.Series(a).prod()`."""
     out = a[0]
     for i in range(1, a.shape[0]):
-        if ~np.isnan(a[i]):
+        if not np.isnan(a[i]):
             if np.isnan(out):
                 out = a[i]
             else:
@@ -220,7 +237,7 @@ def cumsum_1d_nb(a):
     out = np.empty_like(a, dtype=np.float_)
     cumsum = 0
     for i in range(a.shape[0]):
-        if ~np.isnan(a[i]):
+        if not np.isnan(a[i]):
             cumsum += a[i]
             out[i] = cumsum
         else:
@@ -245,7 +262,7 @@ def cumprod_1d_nb(a):
     out = np.empty_like(a, dtype=np.float_)
     cumprod = 1
     for i in range(a.shape[0]):
-        if ~np.isnan(a[i]):
+        if not np.isnan(a[i]):
             cumprod *= a[i]
             out[i] = cumprod
         else:
@@ -336,30 +353,6 @@ def nanstd_nb(a, ddof=0):
     return out
 
 
-# ############# Range functions ############# #
-
-
-@njit(cache=True)
-def concat_ranges_1d_nb(a, start_idxs, end_idxs):
-    """Roll a window.
-
-    For each index pair from `start_idxs` and `end_idxs`, slices `a` along index and concatenates."""
-    out = np.empty((end_idxs[0] - start_idxs[0], start_idxs.shape[0]), dtype=a.dtype)
-    for idx in range(start_idxs.shape[0]):
-        out[:, idx] = a[start_idxs[idx]:end_idxs[idx]]
-    return out
-
-
-@njit(cache=True)
-def concat_ranges_nb(a, start_idxs, end_idxs):
-    """2-dim version of `range_1d_nb`."""
-    out = np.empty((end_idxs[0] - start_idxs[0], start_idxs.shape[0] * a.shape[1]), dtype=a.dtype)
-    for col in range(a.shape[1]):
-        out[:, col * start_idxs.shape[0]:(col + 1) * start_idxs.shape[0]] = \
-            concat_ranges_1d_nb(a[:, col], start_idxs, end_idxs)
-    return out
-
-
 # ############# Rolling functions ############# #
 
 
@@ -492,7 +485,6 @@ def rolling_std_1d_nb(a, window, minp=None, ddof=0):
     cumsum_sq = 0
     nancnt_arr = np.zeros_like(a)
     nancnt = 0
-    mean = 0
     for i in range(a.shape[0]):
         if np.isnan(a[i]):
             nancnt = nancnt + 1
@@ -529,7 +521,7 @@ def rolling_std_nb(a, window, minp=None, ddof=0):
 
 
 @njit(cache=True)
-def ewm_mean_1d_nb(a, span, minp=None, adjust=False):
+def ewm_mean_1d_nb(a, span, minp=0, adjust=False):
     """Return exponential weighted average.
 
     Numba equivalent to `pd.Series(a).ewm(span=span, min_periods=minp, adjust=adjust).mean()`.
@@ -574,7 +566,7 @@ def ewm_mean_1d_nb(a, span, minp=None, adjust=False):
 
 
 @njit(cache=True)
-def ewm_mean_nb(a, span, minp=None, adjust=False):
+def ewm_mean_nb(a, span, minp=0, adjust=False):
     """2-dim version of `ewm_mean_1d_nb`."""
     out = np.empty_like(a, dtype=np.float_)
     for col in range(a.shape[1]):
@@ -583,7 +575,7 @@ def ewm_mean_nb(a, span, minp=None, adjust=False):
 
 
 @njit(cache=True)
-def ewm_std_1d_nb(a, span, minp=None, adjust=False, ddof=0):
+def ewm_std_1d_nb(a, span, minp=0, adjust=False, ddof=0):
     """Return exponential weighted standard deviation.
 
     Numba equivalent to `pd.Series(a).ewm(span=span, min_periods=minp).std(ddof=ddof)`.
@@ -664,7 +656,7 @@ def ewm_std_1d_nb(a, span, minp=None, adjust=False, ddof=0):
 
 
 @njit(cache=True)
-def ewm_std_nb(a, span, minp=None, adjust=False, ddof=0):
+def ewm_std_nb(a, span, minp=0, adjust=False, ddof=0):
     """2-dim version of `ewm_std_1d_nb`."""
     out = np.empty_like(a, dtype=np.float_)
     for col in range(a.shape[1]):
@@ -686,7 +678,7 @@ def expanding_min_1d_nb(a, minp=1):
     for i in range(a.shape[0]):
         if np.isnan(minv) or a[i] < minv:
             minv = a[i]
-        if ~np.isnan(a[i]):
+        if not np.isnan(a[i]):
             cnt += 1
         if cnt < minp:
             out[i] = np.nan
@@ -715,7 +707,7 @@ def expanding_max_1d_nb(a, minp=1):
     for i in range(a.shape[0]):
         if np.isnan(maxv) or a[i] > maxv:
             maxv = a[i]
-        if ~np.isnan(a[i]):
+        if not np.isnan(a[i]):
             cnt += 1
         if cnt < minp:
             out[i] = np.nan
@@ -765,42 +757,93 @@ def expanding_std_nb(a, minp=1, ddof=0):
 
 
 @njit
-def rolling_apply_nb(a, window, apply_func_nb, *args):
+def apply_along_0_nb(a, apply_func_nb, *args):
+    """Apply function on each column.
+
+    `apply_func_nb` should accept index of the column, the array, and `*args`."""
+    out = np.empty_like(a, dtype=np.float_)
+    for col in range(a.shape[1]):
+        out[:, col] = apply_func_nb(col, a[:, col], *args)
+    return out
+
+
+@njit
+def apply_along_1_nb(a, apply_func_nb, *args):
+    """Apply function on each row.
+
+    `apply_func_nb` should accept index of the row, the array, and `*args`."""
+    out = np.empty_like(a, dtype=np.float_)
+    for i in range(a.shape[0]):
+        out[i, :] = apply_func_nb(i, a[i, :], *args)
+    return out
+
+
+@njit
+def rolling_apply_nb(a, window, minp, apply_func_nb, *args):
     """Provide rolling window calculations.
 
     `apply_func_nb` should accept index of the row, index of the column,
     the array, and `*args`. Should return a single value."""
+    if minp is None:
+        minp = window
     out = np.empty_like(a, dtype=np.float_)
+    nancnt_arr = np.empty((a.shape[0],), dtype=np.int_)
     for col in range(a.shape[1]):
+        nancnt = 0
         for i in range(a.shape[0]):
-            window_a = a[max(0, i + 1 - window):i + 1, col]
-            out[i, col] = apply_func_nb(i, col, window_a, *args)
+            if np.isnan(a[i, col]):
+                nancnt = nancnt + 1
+            nancnt_arr[i] = nancnt
+            if i < window:
+                valid_cnt = i + 1 - nancnt
+            else:
+                valid_cnt = window - (nancnt - nancnt_arr[i - window])
+            if valid_cnt < minp:
+                out[i, col] = np.nan
+            else:
+                window_a = a[max(0, i + 1 - window):i + 1, col]
+                out[i, col] = apply_func_nb(i, col, window_a, *args)
     return out
 
 
 @njit
-def rolling_apply_matrix_nb(a, window, apply_func_nb, *args):
+def rolling_matrix_apply_nb(a, window, minp, apply_func_nb, *args):
     """`rolling_apply_nb` with `apply_func_nb` being applied on all columns at once.
 
     `apply_func_nb` should accept index of the row, the 2-dim array, and `*args`.
     Should return a single value or an array of shape `a.shape[1]`."""
+    if minp is None:
+        minp = window
     out = np.empty_like(a, dtype=np.float_)
+    nancnt_arr = np.empty((a.shape[0],), dtype=np.int_)
     for i in range(a.shape[0]):
-        window_a = a[max(0, i + 1 - window):i + 1, :]
-        out[i, :] = apply_func_nb(i, window_a, *args)
+        nancnt = 0
+        for col in range(a.shape[1]):
+            if np.isnan(a[i, col]):
+                nancnt = nancnt + 1
+        nancnt_arr[i] = nancnt
+        if i < window:
+            valid_cnt = i + 1 - nancnt
+        else:
+            valid_cnt = window - (nancnt - nancnt_arr[i - window])
+        if valid_cnt < minp:
+            out[i, :] = np.nan
+        else:
+            window_a = a[max(0, i + 1 - window):i + 1, :]
+            out[i, :] = apply_func_nb(i, window_a, *args)
     return out
 
 
 @njit
-def expanding_apply_nb(a, apply_func_nb, *args):
+def expanding_apply_nb(a, minp, apply_func_nb, *args):
     """Expanding version of `rolling_apply_nb`."""
-    return rolling_apply_nb(a, a.shape[0], apply_func_nb, *args)
+    return rolling_apply_nb(a, a.shape[0], minp, apply_func_nb, *args)
 
 
 @njit
-def expanding_apply_matrix_nb(a, apply_func_nb, *args):
-    """Expanding version of `rolling_apply_matrix_nb`."""
-    return rolling_apply_matrix_nb(a, a.shape[0], apply_func_nb, *args)
+def expanding_matrix_apply_nb(a, minp, apply_func_nb, *args):
+    """Expanding version of `rolling_matrix_apply_nb`."""
+    return rolling_matrix_apply_nb(a, a.shape[0], minp, apply_func_nb, *args)
 
 
 @njit
@@ -839,8 +882,8 @@ def groupby_apply_matrix_nb(a, groups, apply_func_nb, *args):
 def applymap_nb(a, map_func_nb, *args):
     """Map non-NA elements elementwise using `map_func_nb`.
 
-    `map_func_nb` should accept index of the row, index of the column,
-    the element itself, and `*args`. Should return an array of same size."""
+    `map_func_nb` should accept the element itself, and `*args`.
+    Should return an array of same size."""
     out = np.full_like(a, np.nan, dtype=np.float_)
 
     for col in range(out.shape[1]):
@@ -1034,7 +1077,7 @@ def flatten_grouped_nb(a, group_lens, in_c_order):
 
 
 @njit(cache=True)
-def nst_reduce_nb(col, a, n, *args):
+def nst_reduce_nb(col, a, n):
     """Return nst element."""
     if n >= a.shape[0]:
         raise ValueError("index is out of bounds")
@@ -1042,49 +1085,49 @@ def nst_reduce_nb(col, a, n, *args):
 
 
 @njit(cache=True)
-def min_reduce_nb(col, a, *args):
+def min_reduce_nb(col, a):
     """Return min (ignores NaNs)."""
     return np.nanmin(a)
 
 
 @njit(cache=True)
-def max_reduce_nb(col, a, *args):
+def max_reduce_nb(col, a):
     """Return max (ignores NaNs)."""
     return np.nanmax(a)
 
 
 @njit(cache=True)
-def mean_reduce_nb(col, a, *args):
+def mean_reduce_nb(col, a):
     """Return mean (ignores NaNs)."""
     return np.nanmean(a)
 
 
 @njit(cache=True)
-def median_reduce_nb(col, a, *args):
+def median_reduce_nb(col, a):
     """Return median (ignores NaNs)."""
     return np.nanmedian(a)
 
 
 @njit(cache=True)
-def sum_reduce_nb(col, a, *args):
+def sum_reduce_nb(col, a):
     """Return sum (ignores NaNs)."""
     return np.nansum(a)
 
 
 @njit(cache=True)
-def count_reduce_nb(col, a, *args):
+def count_reduce_nb(col, a):
     """Return count (ignores NaNs)."""
     return np.sum(~np.isnan(a))
 
 
 @njit(cache=True)
-def std_reduce_nb(col, a, ddof, *args):
+def std_reduce_nb(col, a, ddof):
     """Return std (ignores NaNs)."""
     return nanstd_1d_nb(a, ddof=ddof)
 
 
 @njit(cache=True)
-def argmin_reduce_nb(col, a, *args):
+def argmin_reduce_nb(col, a):
     """Return position of min."""
     a = np.copy(a)
     mask = np.isnan(a)
@@ -1095,7 +1138,7 @@ def argmin_reduce_nb(col, a, *args):
 
 
 @njit(cache=True)
-def argmax_reduce_nb(col, a, *args):
+def argmax_reduce_nb(col, a):
     """Return position of max."""
     a = np.copy(a)
     mask = np.isnan(a)
@@ -1106,7 +1149,7 @@ def argmax_reduce_nb(col, a, *args):
 
 
 @njit(cache=True)
-def describe_reduce_nb(col, a, perc, ddof, *args):
+def describe_reduce_nb(col, a, perc, ddof):
     """Return descriptive statistics (ignores NaNs).
 
     Numba equivalent to `pd.Series(a).describe(perc)`."""
