@@ -1,4 +1,9 @@
-"""Base class for working with order records."""
+"""Base class for working with order records.
+
+Class `Orders` wraps order records and the corresponding time series (such as open or close)
+to analyze orders. Orders are mainly populated when simulating a portfolio and can be
+accessed as `vectorbt.portfolio.base.Portfolio.orders`.
+"""
 
 import numpy as np
 import pandas as pd
@@ -7,7 +12,7 @@ import plotly.graph_objects as go
 from vectorbt.utils.decorators import cached_property, cached_method
 from vectorbt.utils.colors import adjust_lightness
 from vectorbt.utils.enum import to_value_map
-from vectorbt.utils.widgets import CustomFigureWidget
+from vectorbt.utils.widgets import FigureWidget
 from vectorbt.utils.config import merge_dicts
 from vectorbt.base.reshape_fns import to_1d, to_2d, broadcast_to
 from vectorbt.records.base import Records
@@ -49,10 +54,10 @@ class Orders(Records):
         if not all(field in records_arr.dtype.names for field in order_dt.names):
             raise TypeError("Records array must match order_dt")
 
-    def _indexing_func_meta(self, pd_indexing_func):
+    def _indexing_func_meta(self, pd_indexing_func, **kwargs):
         """Perform indexing on `Orders` and return metadata."""
         new_wrapper, new_records_arr, group_idxs, col_idxs = \
-            Records._indexing_func_meta(self, pd_indexing_func)
+            Records._indexing_func_meta(self, pd_indexing_func, **kwargs)
         new_close = new_wrapper.wrap(to_2d(self.close, raw=True)[:, col_idxs], group_by=False)
         return self.copy(
             wrapper=new_wrapper,
@@ -60,9 +65,9 @@ class Orders(Records):
             close=new_close
         ), group_idxs, col_idxs
 
-    def _indexing_func(self, pd_indexing_func):
+    def _indexing_func(self, pd_indexing_func, **kwargs):
         """Perform indexing on `Orders`."""
-        return self._indexing_func_meta(pd_indexing_func)[0]
+        return self._indexing_func_meta(pd_indexing_func, **kwargs)[0]
 
     @property
     def close(self):
@@ -114,11 +119,12 @@ class Orders(Records):
         return self.filter_by_mask(filter_mask)
 
     @cached_method
-    def buy_rate(self, group_by=None, **kwargs):
+    def buy_rate(self, group_by=None, wrap_kwargs=None):
         """Rate of buy operations."""
         buy_count = to_1d(self.buy.count(group_by=group_by), raw=True)
         total_count = to_1d(self.count(group_by=group_by), raw=True)
-        return self.wrapper.wrap_reduced(buy_count / total_count, group_by=group_by, **kwargs)
+        wrap_kwargs = merge_dicts(dict(name_or_index='buy_rate'), wrap_kwargs)
+        return self.wrapper.wrap_reduced(buy_count / total_count, group_by=group_by, **wrap_kwargs)
 
     @cached_property
     def sell(self):
@@ -127,11 +133,12 @@ class Orders(Records):
         return self.filter_by_mask(filter_mask)
 
     @cached_method
-    def sell_rate(self, group_by=None, **kwargs):
+    def sell_rate(self, group_by=None, wrap_kwargs=None):
         """Rate of sell operations."""
         sell_count = to_1d(self.sell.count(group_by=group_by), raw=True)
         total_count = to_1d(self.count(group_by=group_by), raw=True)
-        return self.wrapper.wrap_reduced(sell_count / total_count, group_by=group_by, **kwargs)
+        wrap_kwargs = merge_dicts(dict(name_or_index='sell_rate'), wrap_kwargs)
+        return self.wrapper.wrap_reduced(sell_count / total_count, group_by=group_by, **wrap_kwargs)
 
     # ############# Plotting ############# #
 
@@ -141,7 +148,7 @@ class Orders(Records):
              close_trace_kwargs=None,
              buy_trace_kwargs=None,
              sell_trace_kwargs=None,
-             row=None, col=None,
+             add_trace_kwargs=None,
              fig=None,
              **layout_kwargs):  # pragma: no cover
         """Plot orders.
@@ -152,8 +159,7 @@ class Orders(Records):
             close_trace_kwargs (dict): Keyword arguments passed to `plotly.graph_objects.Scatter` for `Orders.close`.
             buy_trace_kwargs (dict): Keyword arguments passed to `plotly.graph_objects.Scatter` for "Buy" markers.
             sell_trace_kwargs (dict): Keyword arguments passed to `plotly.graph_objects.Scatter` for "Sell" markers.
-            row (int): Row position.
-            col (int): Column position.
+            add_trace_kwargs (dict): Keyword arguments passed to `add_trace`.
             fig (plotly.graph_objects.Figure): Figure to add traces to.
             **layout_kwargs: Keyword arguments for layout.
 
@@ -178,14 +184,16 @@ class Orders(Records):
             buy_trace_kwargs = {}
         if sell_trace_kwargs is None:
             sell_trace_kwargs = {}
+        if add_trace_kwargs is None:
+            add_trace_kwargs = {}
 
         if fig is None:
-            fig = CustomFigureWidget()
+            fig = FigureWidget()
         fig.update_layout(**layout_kwargs)
 
         # Plot close
         if plot_close:
-            fig = self_col.close.vbt.plot(trace_kwargs=close_trace_kwargs, row=row, col=col, fig=fig)
+            fig = self_col.close.vbt.plot(trace_kwargs=close_trace_kwargs, add_trace_kwargs=add_trace_kwargs, fig=fig)
 
         if len(self_col.values) > 0:
             # Extract information
@@ -221,7 +229,7 @@ class Orders(Records):
                               "<br>Fees: %{customdata[2]:.6f}"
             )
             buy_scatter.update(**buy_trace_kwargs)
-            fig.add_trace(buy_scatter, row=row, col=col)
+            fig.add_trace(buy_scatter, **add_trace_kwargs)
 
             # Plot Sell markers
             sell_mask = side == OrderSide.Sell
@@ -248,6 +256,6 @@ class Orders(Records):
                               "<br>Fees: %{customdata[2]:.6f}"
             )
             sell_scatter.update(**sell_trace_kwargs)
-            fig.add_trace(sell_scatter, row=row, col=col)
+            fig.add_trace(sell_scatter, **add_trace_kwargs)
 
         return fig
