@@ -1,9 +1,32 @@
 """Custom pandas accessors.
 
+Methods can be accessed as follows:
+
+* `SignalsSRAccessor` -> `pd.Series.vbt.signals.*`
+* `SignalsDFAccessor` -> `pd.DataFrame.vbt.signals.*`
+
+```python-repl
+>>> import pandas as pd
+>>> import vectorbt as vbt
+
+>>> # vectorbt.signals.accessors.SignalsAccessor.rank
+>>> pd.Series([False, True, True, True, False]).vbt.signals.rank()
+0    0
+1    1
+2    2
+3    3
+4    0
+dtype: int64
+```
+
+The accessors extend `vectorbt.generic.accessors`.
+
 !!! note
     The underlying Series/DataFrame should already be a signal series.
 
     Input arrays should be `np.bool`.
+
+Run for the examples below:
     
 ```python-repl
 >>> import vectorbt as vbt
@@ -34,23 +57,23 @@
 
 import numpy as np
 import pandas as pd
-import plotly.graph_objects as go
 
 from vectorbt.root_accessors import register_dataframe_accessor, register_series_accessor
 from vectorbt.utils import checks
+from vectorbt.utils.decorators import class_or_instancemethod
 from vectorbt.utils.config import merge_dicts
 from vectorbt.utils.colors import adjust_lightness
-from vectorbt.utils.widgets import CustomFigureWidget
 from vectorbt.base import reshape_fns
 from vectorbt.base.class_helpers import add_nb_methods
-from vectorbt.generic.accessors import Generic_Accessor, Generic_SRAccessor, Generic_DFAccessor
+from vectorbt.base.array_wrapper import ArrayWrapper
+from vectorbt.generic.accessors import GenericAccessor, GenericSRAccessor, GenericDFAccessor
 from vectorbt.signals import nb
 
 
 @add_nb_methods([
-    nb.fshift_nb,
+    (nb.fshift_nb, False),
 ], module_name='vectorbt.signals.nb')
-class Signals_Accessor(Generic_Accessor):
+class SignalsAccessor(GenericAccessor):
     """Accessor on top of signal series. For both, Series and DataFrames.
 
     Accessible through `pd.Series.vbt.signals` and `pd.DataFrame.vbt.signals`."""
@@ -61,11 +84,11 @@ class Signals_Accessor(Generic_Accessor):
 
         checks.assert_dtype(obj, np.bool)
 
-        Generic_Accessor.__init__(self, obj, **kwargs)
+        GenericAccessor.__init__(self, obj, **kwargs)
 
     @classmethod
     def empty(cls, *args, fill_value=False, **kwargs):
-        """`vectorbt.base.accessors.Base_Accessor.empty` with `fill_value=False`.
+        """`vectorbt.base.accessors.BaseAccessor.empty` with `fill_value=False`.
 
         ## Example
 
@@ -87,11 +110,11 @@ class Signals_Accessor(Generic_Accessor):
         2020-01-05  False  False  False
         ```
         """
-        return Generic_Accessor.empty(*args, fill_value=fill_value, dtype=bool, **kwargs)
+        return GenericAccessor.empty(*args, fill_value=fill_value, dtype=bool, **kwargs)
 
     @classmethod
     def empty_like(cls, *args, fill_value=False, **kwargs):
-        """`vectorbt.base.accessors.Base_Accessor.empty_like` with `fill_value=False`.
+        """`vectorbt.base.accessors.BaseAccessor.empty_like` with `fill_value=False`.
 
         ## Example
 
@@ -113,9 +136,9 @@ class Signals_Accessor(Generic_Accessor):
         2020-01-05  False  False  False
         ```
         """
-        return Generic_Accessor.empty_like(*args, fill_value=fill_value, dtype=bool, **kwargs)
+        return GenericAccessor.empty_like(*args, fill_value=fill_value, dtype=bool, **kwargs)
 
-    # ############# Signal generation ############# #
+    # ############# Generation ############# #
 
     @classmethod
     def generate(cls, shape, choice_func_nb, *args, **kwargs):
@@ -226,7 +249,7 @@ class Signals_Accessor(Generic_Accessor):
             return pd.Series(result1[:, 0], **kwargs), pd.Series(result2[:, 0], **kwargs)
         return pd.DataFrame(result1, **kwargs), pd.DataFrame(result2, **kwargs)
 
-    def generate_exits(self, exit_choice_func_nb, *args, wait=1):
+    def generate_exits(self, exit_choice_func_nb, *args, wait=1, wrap_kwargs=None):
         """See `vectorbt.signals.nb.generate_ex_nb`.
 
         ## Example
@@ -249,7 +272,40 @@ class Signals_Accessor(Generic_Accessor):
         """
         checks.assert_numba_func(exit_choice_func_nb)
 
-        return self.wrapper.wrap(nb.generate_ex_nb(self.to_2d_array(), wait, exit_choice_func_nb, *args))
+        exits = nb.generate_ex_nb(self.to_2d_array(), wait, exit_choice_func_nb, *args)
+        return self.wrapper.wrap(exits, **merge_dicts({}, wrap_kwargs))
+
+    # ############# Filtering ############# #
+
+    @class_or_instancemethod
+    def clean(self_or_cls, *args, entry_first=True, broadcast_kwargs=None, wrap_kwargs=None):
+        """Clean signals.
+
+        If one array passed, see `SignalsAccessor.first`.
+        If two arrays passed, entries and exits, see `vectorbt.signals.nb.clean_enex_nb`."""
+        if not isinstance(self_or_cls, type):
+            args = (self_or_cls._obj, *args)
+        if len(args) == 1:
+            obj = args[0]
+            if not isinstance(obj, (pd.Series, pd.DataFrame)):
+                wrapper = ArrayWrapper.from_shape(np.asarray(obj).shape)
+                obj = wrapper.wrap(obj, **merge_dicts({}, wrap_kwargs))
+            return obj.vbt.signals.first()
+        elif len(args) == 2:
+            if broadcast_kwargs is None:
+                broadcast_kwargs = {}
+            entries, exits = reshape_fns.broadcast(*args, **broadcast_kwargs)
+            entries_out, exits_out = nb.clean_enex_nb(
+                entries.vbt.to_2d_array(),
+                exits.vbt.to_2d_array(),
+                entry_first
+            )
+            return (
+                entries.vbt.wrapper.wrap(entries_out, **merge_dicts({}, wrap_kwargs)),
+                exits.vbt.wrapper.wrap(exits_out, **merge_dicts({}, wrap_kwargs))
+            )
+        else:
+            raise ValueError("Either one or two arrays must be passed")
 
     # ############# Random ############# #
 
@@ -260,8 +316,8 @@ class Signals_Accessor(Generic_Accessor):
         If `n` is set, see `vectorbt.signals.nb.generate_rand_nb`.
         If `prob` is set, see `vectorbt.signals.nb.generate_rand_by_prob_nb`.
 
-        `n` should be either a scalar or an array that will be broadcast to the number of columns.
-        `prob` should be either a single number or an array that will be broadcast to match `shape`.
+        `n` should be either a scalar or an array that will broadcast to the number of columns.
+        `prob` should be either a single number or an array that will broadcast to match `shape`.
         `**kwargs` will be passed to pandas constructor.
 
         ## Example
@@ -298,6 +354,8 @@ class Signals_Accessor(Generic_Accessor):
             flex_2d = False
             shape = (shape[0], 1)
 
+        if n is not None and prob is not None:
+            raise ValueError("Either n or prob should be set")
         if n is not None:
             n = np.broadcast_to(n, shape[1])
             result = nb.generate_rand_nb(shape, n, seed=seed)
@@ -323,7 +381,7 @@ class Signals_Accessor(Generic_Accessor):
         If `n` is set, see `vectorbt.signals.nb.generate_rand_enex_nb`.
         If `entry_prob` and `exit_prob` are set, see `vectorbt.signals.nb.generate_rand_enex_by_prob_nb`.
 
-        For arguments, see `Signals_Accessor.generate_random`.
+        For arguments, see `SignalsAccessor.generate_random`.
 
         ## Example
 
@@ -393,7 +451,7 @@ class Signals_Accessor(Generic_Accessor):
             return pd.Series(entries[:, 0], **kwargs), pd.Series(exits[:, 0], **kwargs)
         return pd.DataFrame(entries, **kwargs), pd.DataFrame(exits, **kwargs)
 
-    def generate_random_exits(self, prob=None, seed=None, wait=1):
+    def generate_random_exits(self, prob=None, seed=None, wait=1, wrap_kwargs=None):
         """Generate exit signals randomly.
 
         If `prob` is None, see `vectorbt.signals.nb.generate_rand_ex_nb`.
@@ -425,19 +483,21 @@ class Signals_Accessor(Generic_Accessor):
         """
         if prob is not None:
             obj, prob = reshape_fns.broadcast(self._obj, prob, keep_raw=[False, True])
-            return obj.vbt.wrapper.wrap(nb.generate_rand_ex_by_prob_nb(
-                obj.vbt.to_2d_array(), prob, wait, obj.ndim == 2, seed=seed))
-        return self.wrapper.wrap(nb.generate_rand_ex_nb(self.to_2d_array(), wait, seed=seed))
+            exits = nb.generate_rand_ex_by_prob_nb(obj.vbt.to_2d_array(), prob, wait, obj.ndim == 2, seed=seed)
+            return obj.vbt.wrapper.wrap(exits, **merge_dicts({}, wrap_kwargs))
+        exits = nb.generate_rand_ex_nb(self.to_2d_array(), wait, seed=seed)
+        return self.wrapper.wrap(exits, **merge_dicts({}, wrap_kwargs))
 
     def generate_stop_exits(self, ts, stop, trailing=False, entry_wait=1, exit_wait=1,
-                            first=True, iteratively=False, broadcast_kwargs=None):
+                            first=True, iteratively=False, broadcast_kwargs=None, wrap_kwargs=None):
         """Generate exits based on when `ts` hits the stop.
 
+        For arguments, see `vectorbt.signals.nb.stop_choice_nb`.
         If `iteratively` is True, see `vectorbt.signals.nb.generate_stop_ex_iter_nb`.
         Otherwise, see `vectorbt.signals.nb.generate_stop_ex_nb`.
 
-        Arguments `entries`, `ts` and `stop` will be broadcast using
-        `vectorbt.base.reshape_fns.broadcast` with `broadcast_kwargs`.
+        Arguments `entries`, `ts` and `stop` will broadcast using `vectorbt.base.reshape_fns.broadcast`
+        with `broadcast_kwargs`.
 
         For arguments, see `vectorbt.signals.nb.stop_choice_nb`.
 
@@ -477,29 +537,35 @@ class Signals_Accessor(Generic_Accessor):
         if iteratively:
             new_entries, exits = nb.generate_stop_ex_iter_nb(
                 entries.vbt.to_2d_array(), ts, stop, trailing, entry_wait, exit_wait, entries.ndim == 2)
-            return entries.vbt.wrapper.wrap(new_entries), entries.vbt.wrapper.wrap(exits)
+            return entries.vbt.wrapper.wrap(new_entries, **merge_dicts({}, wrap_kwargs)), \
+                   entries.vbt.wrapper.wrap(exits, **merge_dicts({}, wrap_kwargs))
         else:
             exits = nb.generate_stop_ex_nb(
                 entries.vbt.to_2d_array(), ts, stop, trailing, exit_wait, first, entries.ndim == 2)
-            return entries.vbt.wrapper.wrap(exits)
+            return entries.vbt.wrapper.wrap(exits, **merge_dicts({}, wrap_kwargs))
 
-    def generate_adv_stop_exits(self, open, high=None, low=None, close=None, is_open_safe=True,
+    def generate_ohlc_stop_exits(self, open, high=None, low=None, close=None, is_open_safe=True,
                                 out_dict=None, sl_stop=0., ts_stop=0., tp_stop=0., entry_wait=1,
-                                exit_wait=1, first=True, iteratively=False, broadcast_kwargs=None):
-        """Generate exits based on when price hits (trailing) stop loss or take profit.
+                                exit_wait=1, first=True, iteratively=False, broadcast_kwargs=None, wrap_kwargs=None):
+        """Generate exits based on when the price hits (trailing) stop loss or take profit.
 
         If any of `high`, `low` or `close` is None, it will be set to `open`.
 
         Use `out_dict` as a dict to pass `hit_price` and `stop_type` arrays. You can also
         set `out_dict` to {} to produce these arrays automatically and still have access to them.
 
-        If `iteratively` is True, see `vectorbt.signals.nb.generate_adv_stop_ex_iter_nb`.
-        Otherwise, see `vectorbt.signals.nb.generate_adv_stop_ex_nb`.
+        For arguments, see `vectorbt.signals.nb.ohlc_stop_choice_nb`.
+        If `iteratively` is True, see `vectorbt.signals.nb.generate_ohlc_stop_ex_iter_nb`.
+        Otherwise, see `vectorbt.signals.nb.generate_ohlc_stop_ex_nb`.
 
-        All array-like arguments including stops and `out_dict` will be broadcast using
+        All array-like arguments including stops and `out_dict` will broadcast using
         `vectorbt.base.reshape_fns.broadcast` with `broadcast_kwargs`.
 
-        For arguments, see `vectorbt.signals.nb.adv_stop_choice_nb`.
+        For arguments, see `vectorbt.signals.nb.ohlc_stop_choice_nb`.
+
+        !!! note
+            `open` isn't necessarily open price, but can be any entry price (even previous close).
+            Stop price is calculated based solely upon `open`.
 
         ## Example
 
@@ -513,7 +579,7 @@ class Signals_Accessor(Generic_Accessor):
         ...     'close': [10, 11, 12, 11, 10]
         ... })
         >>> out_dict = {}
-        >>> exits = sig.vbt.signals.generate_adv_stop_exits(
+        >>> exits = sig.vbt.signals.generate_ohlc_stop_exits(
         ...     price['open'], price['high'], price['low'], price['close'],
         ...     out_dict=out_dict, sl_stop=0.2, ts_stop=0.2, tp_stop=0.2)
         >>> out_dict['hit_price'][~exits] = np.nan
@@ -585,30 +651,31 @@ class Signals_Accessor(Generic_Accessor):
 
         # Perform generation
         if iteratively:
-            new_entries, exits = nb.generate_adv_stop_ex_iter_nb(
+            new_entries, exits = nb.generate_ohlc_stop_ex_iter_nb(
                 entries.vbt.to_2d_array(), open, high, low, close, hit_price_out,
                 stop_type_out, sl_stop, ts_stop, tp_stop, is_open_safe, entry_wait,
                 exit_wait, first, entries.ndim == 2)
-            out_dict['hit_price'] = entries.vbt.wrapper.wrap(hit_price_out)
-            out_dict['stop_type'] = entries.vbt.wrapper.wrap(stop_type_out)
-            return entries.vbt.wrapper.wrap(new_entries), entries.vbt.wrapper.wrap(exits)
+            out_dict['hit_price'] = entries.vbt.wrapper.wrap(hit_price_out, **merge_dicts({}, wrap_kwargs))
+            out_dict['stop_type'] = entries.vbt.wrapper.wrap(stop_type_out, **merge_dicts({}, wrap_kwargs))
+            return entries.vbt.wrapper.wrap(new_entries, **merge_dicts({}, wrap_kwargs)), \
+                   entries.vbt.wrapper.wrap(exits, **merge_dicts({}, wrap_kwargs))
         else:
-            exits = nb.generate_adv_stop_ex_nb(
+            exits = nb.generate_ohlc_stop_ex_nb(
                 entries.vbt.to_2d_array(), open, high, low, close, hit_price_out,
                 stop_type_out, sl_stop, ts_stop, tp_stop, is_open_safe, exit_wait,
                 first, entries.ndim == 2)
-            out_dict['hit_price'] = entries.vbt.wrapper.wrap(hit_price_out)
-            out_dict['stop_type'] = entries.vbt.wrapper.wrap(stop_type_out)
-            return entries.vbt.wrapper.wrap(exits)
+            out_dict['hit_price'] = entries.vbt.wrapper.wrap(hit_price_out, **merge_dicts({}, wrap_kwargs))
+            out_dict['stop_type'] = entries.vbt.wrapper.wrap(stop_type_out, **merge_dicts({}, wrap_kwargs))
+            return entries.vbt.wrapper.wrap(exits, **merge_dicts({}, wrap_kwargs))
 
     # ############# Map and reduce ############# #
 
     def map_reduce_between(self, other=None, map_func_nb=None, map_args=None,
-                           reduce_func_nb=None, reduce_args=None, broadcast_kwargs=None):
+                           reduce_func_nb=None, reduce_args=None, broadcast_kwargs=None, wrap_kwargs=None):
         """See `vectorbt.signals.nb.map_reduce_between_nb`.
 
         If `other` specified, see `vectorbt.signals.nb.map_reduce_between_two_nb`.
-        Both will be broadcast using `vectorbt.base.reshape_fns.broadcast`
+        Both will broadcast using `vectorbt.base.reshape_fns.broadcast`
         with `broadcast_kwargs`.
 
         Note that `map_args` and `reduce_args` won't be broadcast.
@@ -640,6 +707,7 @@ class Signals_Accessor(Generic_Accessor):
         if reduce_args is None:
             reduce_args = ()
 
+        wrap_kwargs = merge_dicts(dict(name_or_index='map_reduce_between'), wrap_kwargs)
         if other is None:
             # One input array
             result = nb.map_reduce_between_nb(
@@ -647,9 +715,7 @@ class Signals_Accessor(Generic_Accessor):
                 map_func_nb, map_args,
                 reduce_func_nb, reduce_args
             )
-            if isinstance(self._obj, pd.Series):
-                return result[0]
-            return pd.Series(result, index=self.wrapper.columns)
+            return self.wrapper.wrap_reduced(result, **wrap_kwargs)
         else:
             # Two input arrays
             obj, other = reshape_fns.broadcast(self._obj, other, **broadcast_kwargs)
@@ -660,10 +726,10 @@ class Signals_Accessor(Generic_Accessor):
                 map_func_nb, map_args,
                 reduce_func_nb, reduce_args
             )
-            return obj.vbt.wrapper.wrap_reduced(result)
+            return obj.vbt.wrapper.wrap_reduced(result, **wrap_kwargs)
 
     def map_reduce_partitions(self, map_func_nb=None, map_args=None,
-                              reduce_func_nb=None, reduce_args=None):
+                              reduce_func_nb=None, reduce_args=None, wrap_kwargs=None):
         """See `vectorbt.signals.nb.map_reduce_partitions_nb`.
 
         ## Example
@@ -696,26 +762,26 @@ class Signals_Accessor(Generic_Accessor):
             map_func_nb, map_args,
             reduce_func_nb, reduce_args
         )
-        return self.wrapper.wrap_reduced(result)
+        wrap_kwargs = merge_dicts(dict(name_or_index='map_reduce_partitions'), wrap_kwargs)
+        return self.wrapper.wrap_reduced(result, **wrap_kwargs)
 
-    def num_signals(self):
+    def num_signals(self, **kwargs):
         """Sum up True values."""
-        return self.sum()
+        kwargs = merge_dicts(dict(wrap_kwargs=dict(name_or_index='num_signals')), kwargs)
+        return self.sum(**kwargs)
 
     def avg_distance(self, to=None, **kwargs):
         """Calculate the average distance between True values in `self` and optionally `to`.
 
-        See `Signals_Accessor.map_reduce_between`."""
+        See `SignalsAccessor.map_reduce_between`."""
+        kwargs = merge_dicts(dict(wrap_kwargs=dict(name_or_index='avg_distance')), kwargs)
         return self.map_reduce_between(
-            other=to,
-            map_func_nb=nb.distance_map_nb,
-            reduce_func_nb=nb.mean_reduce_nb,
-            **kwargs
-        )
+            other=to, map_func_nb=nb.distance_map_nb,
+            reduce_func_nb=nb.mean_reduce_nb, **kwargs)
 
     # ############# Ranking ############# #
 
-    def rank(self, reset_by=None, after_false=False, allow_gaps=False, broadcast_kwargs=None):
+    def rank(self, reset_by=None, after_false=False, allow_gaps=False, broadcast_kwargs=None, wrap_kwargs=None):
         """See `vectorbt.signals.nb.rank_nb`.
 
         ## Example
@@ -767,9 +833,9 @@ class Signals_Accessor(Generic_Accessor):
             reset_by=reset_by,
             after_false=after_false,
             allow_gaps=allow_gaps)
-        return obj.vbt.wrapper.wrap(ranked)
+        return obj.vbt.wrapper.wrap(ranked, **merge_dicts({}, wrap_kwargs))
 
-    def rank_partitions(self, reset_by=None, after_false=False, broadcast_kwargs=None):
+    def rank_partitions(self, reset_by=None, after_false=False, broadcast_kwargs=None, wrap_kwargs=None):
         """See `vectorbt.signals.nb.rank_partitions_nb`.
 
         ## Example
@@ -812,41 +878,41 @@ class Signals_Accessor(Generic_Accessor):
             obj.vbt.to_2d_array(),
             reset_by=reset_by,
             after_false=after_false)
-        return obj.vbt.wrapper.wrap(ranked)
+        return obj.vbt.wrapper.wrap(ranked, **merge_dicts({}, wrap_kwargs))
 
-    def first(self, **kwargs):
+    def first(self, wrap_kwargs=None, **kwargs):
         """`vectorbt.signals.nb.rank_nb` == 1."""
-        return self.wrapper.wrap(self.rank(**kwargs).values == 1)
+        return self.wrapper.wrap(self.rank(**kwargs).values == 1, **merge_dicts({}, wrap_kwargs))
 
-    def nst(self, n, **kwargs):
+    def nst(self, n, wrap_kwargs=None, **kwargs):
         """`vectorbt.signals.nb.rank_nb` == n."""
-        return self.wrapper.wrap(self.rank(**kwargs).values == n)
+        return self.wrapper.wrap(self.rank(**kwargs).values == n, **merge_dicts({}, wrap_kwargs))
 
-    def from_nst(self, n, **kwargs):
+    def from_nst(self, n, wrap_kwargs=None, **kwargs):
         """`vectorbt.signals.nb.rank_nb` >= n."""
-        return self.wrapper.wrap(self.rank(**kwargs).values >= n)
+        return self.wrapper.wrap(self.rank(**kwargs).values >= n, **merge_dicts({}, wrap_kwargs))
 
     # ############# Logical operations ############# #
 
-    def AND(self, *others, **kwargs):
-        """Combine with each in `*others` using logical AND.
+    def AND(self, other, **kwargs):
+        """Combine with `other` using logical AND.
 
-        See `vectorbt.base.accessors.Base_Accessor.combine_with_multiple`.
+        See `vectorbt.base.accessors.BaseAccessor.combine`.
 
         """
-        return self.combine_with_multiple(others, combine_func=np.logical_and, **kwargs)
+        return self.combine(other, combine_func=np.logical_and, **kwargs)
 
-    def OR(self, *others, **kwargs):
-        """Combine with each in `*others` using logical OR.
+    def OR(self, other, **kwargs):
+        """Combine with `other` using logical OR.
 
-        See `vectorbt.base.accessors.Base_Accessor.combine_with_multiple`.
+        See `vectorbt.base.accessors.BaseAccessor.combine`.
 
         ## Example
 
         Perform two OR operations and concatenate them:
         ```python-repl
         >>> ts = pd.Series([1, 2, 3, 2, 1])
-        >>> sig.vbt.signals.OR(ts > 1, ts > 2, concat=True, keys=['>1', '>2'])
+        >>> sig.vbt.signals.OR([ts > 1, ts > 2], concat=True, keys=['>1', '>2'])
                                     >1                   >2
                         a     b      c      a      b      c
         2020-01-01   True  True   True   True   True   True
@@ -856,211 +922,20 @@ class Signals_Accessor(Generic_Accessor):
         2020-01-05  False  True  False  False   True  False
         ```
         """
-        return self.combine_with_multiple(others, combine_func=np.logical_or, **kwargs)
+        return self.combine(other, combine_func=np.logical_or, **kwargs)
 
-    def XOR(self, *others, **kwargs):
-        """Combine with each in `*others` using logical XOR.
+    def XOR(self, other, **kwargs):
+        """Combine with `other` using logical XOR.
 
-        See `vectorbt.base.accessors.Base_Accessor.combine_with_multiple`."""
-        return self.combine_with_multiple(others, combine_func=np.logical_xor, **kwargs)
+        See `vectorbt.base.accessors.BaseAccessor.combine`."""
+        return self.combine(other, combine_func=np.logical_xor, **kwargs)
 
-
-@register_series_accessor('signals')
-class Signals_SRAccessor(Signals_Accessor, Generic_SRAccessor):
-    """Accessor on top of signal series. For Series only.
-
-    Accessible through `pd.Series.vbt.signals`."""
-
-    def __init__(self, obj, **kwargs):
-        if not checks.is_pandas(obj):  # parent accessor
-            obj = obj._obj
-
-        Generic_SRAccessor.__init__(self, obj, **kwargs)
-        Signals_Accessor.__init__(self, obj, **kwargs)
-
-    def plot(self, name=None, trace_kwargs=None, row=None, col=None, yref='y',
-             fig=None, **layout_kwargs):  # pragma: no cover
-        """Plot Series as a line.
+    def plot(self, yref='y', **kwargs):  # pragma: no cover
+        """Plot signals.
 
         Args:
-            name (str): Name of the signals.
-            trace_kwargs (dict): Keyword arguments passed to `plotly.graph_objects.Scatter`.
-            row (int): Row position.
-            col (int): Column position.
             yref (str): Y coordinate axis.
-            fig (plotly.graph_objects.Figure): Figure to add traces to.
-            **layout_kwargs: Keyword arguments for layout.
-
-        ## Example
-
-        ```python-repl
-        >>> sig['a'].vbt.signals.plot()
-        ```
-
-        ![](/vectorbt/docs/img/signals_sr_plot.png)
-        """
-        if trace_kwargs is None:
-            trace_kwargs = {}
-        # Set up figure
-        if fig is None:
-            fig = CustomFigureWidget()
-        default_layout = dict()
-        default_layout['yaxis' + yref[1:]] = dict(
-            tickmode='array',
-            tickvals=[0, 1],
-            ticktext=['false', 'true']
-        )
-        fig.update_layout(**default_layout)
-        fig.update_layout(**layout_kwargs)
-        if name is None:
-            if 'name' in trace_kwargs:
-                name = trace_kwargs.pop('name')
-            else:
-                name = self._obj.name
-        if name is not None:
-            name = str(name)
-
-        scatter = go.Scatter(
-            x=self.wrapper.index,
-            y=self._obj.values,
-            mode='lines',
-            name=name,
-            showlegend=name is not None
-        )
-        scatter.update(**trace_kwargs)
-        fig.add_trace(scatter, row=row, col=col)
-
-        return fig
-
-    def plot_as_markers(self, y=None, name=None, trace_kwargs=None, row=None, col=None,
-                        fig=None, **layout_kwargs):  # pragma: no cover
-        """Plot Series as markers.
-
-        Args:
-            y (array_like): Y-axis values to plot markers on.
-
-                !!! note
-                    Doesn't plot `y`.
-
-            name (str): Name of the signals.
-            trace_kwargs (dict): Keyword arguments passed to `plotly.graph_objects.Scatter`.
-            row (int): Row position.
-            col (int): Column position.
-            fig (plotly.graph_objects.Figure): Figure to add traces to.
-            **layout_kwargs: Keyword arguments for layout.
-
-        ## Example
-
-        ```python-repl
-        >>> ts = pd.Series([1, 2, 3, 2, 1], index=sig.index)
-        >>> fig = ts.vbt.plot()
-        >>> sig['b'].vbt.signals.plot_as_entry_markers(y=ts, fig=fig)
-        >>> (~sig['b']).vbt.signals.plot_as_exit_markers(y=ts, fig=fig)
-        ```
-
-        ![](/vectorbt/docs/img/signals_plot_as_markers.png)
-        """
-        from vectorbt.settings import contrast_color_schema
-
-        if trace_kwargs is None:
-            trace_kwargs = {}
-
-        if fig is None:
-            fig = CustomFigureWidget()
-        fig.update_layout(**layout_kwargs)
-        if name is None:
-            if 'name' in trace_kwargs:
-                name = trace_kwargs.pop('name')
-            else:
-                name = self._obj.name
-        if name is not None:
-            name = str(name)
-
-        # Plot markers
-        _y = 1 if y is None else y
-        scatter = go.Scatter(
-            x=self.wrapper.index,
-            y=np.where(self._obj, _y, np.nan),
-            mode='markers',
-            marker=dict(
-                symbol='circle',
-                color=contrast_color_schema['blue'],
-                size=7,
-                line=dict(
-                    width=1,
-                    color=adjust_lightness(contrast_color_schema['blue'])
-                )
-            ),
-            name=name,
-            showlegend=name is not None
-        )
-        scatter.update(**trace_kwargs)
-        fig.add_trace(scatter, row=row, col=col)
-        return fig
-
-    def plot_as_entry_markers(self, *args, name='Entry', trace_kwargs=None, **kwargs):  # pragma: no cover
-        """Plot signals as entry markers.
-
-        See `Signals_SRAccessor.plot_as_markers`."""
-        from vectorbt.settings import contrast_color_schema
-
-        if trace_kwargs is None:
-            trace_kwargs = {}
-        trace_kwargs = merge_dicts(dict(
-            marker=dict(
-                symbol='triangle-up',
-                color=contrast_color_schema['green'],
-                size=8,
-                line=dict(
-                    width=1,
-                    color=adjust_lightness(contrast_color_schema['green'])
-                )
-            )
-        ), trace_kwargs)
-        return self.plot_as_markers(*args, name=name, trace_kwargs=trace_kwargs, **kwargs)
-
-    def plot_as_exit_markers(self, *args, name='Exit', trace_kwargs=None, **kwargs):  # pragma: no cover
-        """Plot signals as exit markers.
-
-        See `Signals_SRAccessor.plot_as_markers`."""
-        from vectorbt.settings import contrast_color_schema
-
-        if trace_kwargs is None:
-            trace_kwargs = {}
-        trace_kwargs = merge_dicts(dict(
-            marker=dict(
-                symbol='triangle-down',
-                color=contrast_color_schema['red'],
-                size=8,
-                line=dict(
-                    width=1,
-                    color=adjust_lightness(contrast_color_schema['red'])
-                )
-            )
-        ), trace_kwargs)
-        return self.plot_as_markers(*args, name=name, trace_kwargs=trace_kwargs, **kwargs)
-
-
-@register_dataframe_accessor('signals')
-class Signals_DFAccessor(Signals_Accessor, Generic_DFAccessor):
-    """Accessor on top of signal series. For DataFrames only.
-
-    Accessible through `pd.DataFrame.vbt.signals`."""
-
-    def __init__(self, obj, **kwargs):
-        if not checks.is_pandas(obj):  # parent accessor
-            obj = obj._obj
-
-        Generic_DFAccessor.__init__(self, obj, **kwargs)
-        Signals_Accessor.__init__(self, obj, **kwargs)
-
-    def plot(self, trace_kwargs=None, fig=None, **kwargs):  # pragma: no cover
-        """Plot each column in DataFrame as a line.
-
-        Args:
-            trace_kwargs (dict or list of dict): Keyword arguments passed to each `plotly.graph_objects.Scatter`.
-            fig (plotly.graph_objects.Figure): Figure to add traces to.
-            **kwargs: Keyword arguments passed to `Signals_SRAccessor.plot`.
+            **kwargs: Keyword arguments passed to `vectorbt.generic.accessors.GenericAccessor.lineplot`.
 
         ## Example
 
@@ -1070,13 +945,117 @@ class Signals_DFAccessor(Signals_Accessor, Generic_DFAccessor):
 
         ![](/vectorbt/docs/img/signals_df_plot.png)
         """
-        if trace_kwargs is None:
-            trace_kwargs = {}
-        for col in range(self._obj.shape[1]):
-            fig = self._obj.iloc[:, col].vbt.signals.plot(
-                trace_kwargs=trace_kwargs,
-                fig=fig,
-                **kwargs
-            )
+        default_layout = dict()
+        default_layout['yaxis' + yref[1:]] = dict(
+            tickmode='array',
+            tickvals=[0, 1],
+            ticktext=['false', 'true']
+        )
+        return self._obj.vbt.lineplot(**merge_dicts(default_layout, kwargs))
 
-        return fig
+
+@register_series_accessor('signals')
+class SignalsSRAccessor(SignalsAccessor, GenericSRAccessor):
+    """Accessor on top of signal series. For Series only.
+
+    Accessible through `pd.Series.vbt.signals`."""
+
+    def __init__(self, obj, **kwargs):
+        if not checks.is_pandas(obj):  # parent accessor
+            obj = obj._obj
+
+        GenericSRAccessor.__init__(self, obj, **kwargs)
+        SignalsAccessor.__init__(self, obj, **kwargs)
+
+    def plot_as_markers(self, y=None, **kwargs):  # pragma: no cover
+        """Plot Series as markers.
+
+        Args:
+            y (array_like): Y-axis values to plot markers on.
+            **kwargs: Keyword arguments passed to `vectorbt.generic.accessors.GenericAccessor.scatterplot`.
+
+        ## Example
+
+        ```python-repl
+        >>> ts = pd.Series([1, 2, 3, 2, 1], index=sig.index)
+        >>> fig = ts.vbt.lineplot()
+        >>> sig['b'].vbt.signals.plot_as_entry_markers(y=ts, fig=fig)
+        >>> (~sig['b']).vbt.signals.plot_as_exit_markers(y=ts, fig=fig)
+        ```
+
+        ![](/vectorbt/docs/img/signals_plot_as_markers.png)
+        """
+        from vectorbt.settings import contrast_color_schema
+
+        if y is None:
+            y = pd.Series.vbt.empty_like(self._obj, 1)
+
+        return y[self._obj].vbt.scatterplot(**merge_dicts(dict(
+            trace_kwargs=dict(
+                marker=dict(
+                    symbol='circle',
+                    color=contrast_color_schema['blue'],
+                    size=7,
+                    line=dict(
+                        width=1,
+                        color=adjust_lightness(contrast_color_schema['blue'])
+                    )
+                )
+            )
+        ), kwargs))
+
+    def plot_as_entry_markers(self, y=None, **kwargs):  # pragma: no cover
+        """Plot signals as entry markers.
+
+        See `SignalsSRAccessor.plot_as_markers`."""
+        from vectorbt.settings import contrast_color_schema
+
+        return self.plot_as_markers(y=y, **merge_dicts(dict(
+            trace_kwargs=dict(
+                marker=dict(
+                    symbol='triangle-up',
+                    color=contrast_color_schema['green'],
+                    size=8,
+                    line=dict(
+                        width=1,
+                        color=adjust_lightness(contrast_color_schema['green'])
+                    )
+                ),
+                name='Entry'
+            )
+        ), kwargs))
+
+    def plot_as_exit_markers(self, y=None, **kwargs):  # pragma: no cover
+        """Plot signals as exit markers.
+
+        See `SignalsSRAccessor.plot_as_markers`."""
+        from vectorbt.settings import contrast_color_schema
+
+        return self.plot_as_markers(y=y, **merge_dicts(dict(
+            trace_kwargs=dict(
+                marker=dict(
+                    symbol='triangle-down',
+                    color=contrast_color_schema['red'],
+                    size=8,
+                    line=dict(
+                        width=1,
+                        color=adjust_lightness(contrast_color_schema['red'])
+                    )
+                ),
+                name='Exit'
+            )
+        ), kwargs))
+
+
+@register_dataframe_accessor('signals')
+class SignalsDFAccessor(SignalsAccessor, GenericDFAccessor):
+    """Accessor on top of signal series. For DataFrames only.
+
+    Accessible through `pd.DataFrame.vbt.signals`."""
+
+    def __init__(self, obj, **kwargs):
+        if not checks.is_pandas(obj):  # parent accessor
+            obj = obj._obj
+
+        GenericDFAccessor.__init__(self, obj, **kwargs)
+        SignalsAccessor.__init__(self, obj, **kwargs)
