@@ -1,9 +1,12 @@
 """Utilities for displaying widgets."""
 
 import plotly.graph_objects as go
+from plotly.subplots import make_subplots as _make_subplots
+
+from vectorbt.utils.config import merge_dicts
 
 
-class CustomFigure(go.Figure):
+class Figure(go.Figure):
     def __init__(self, *args, **kwargs):
         """Subclass of the `plotly.graph_objects.Figure` class initialized
         with default parameters from `vectorbt.settings.layout`."""
@@ -11,16 +14,14 @@ class CustomFigure(go.Figure):
 
         layout = kwargs.pop('layout', {})
         super().__init__(*args, **kwargs)
-        self.update_layout(**settings.layout)
-        self.update_layout(colorway=list(settings.color_schema.values()))
-        self.update_layout(**layout)
+        self.update_layout(**merge_dicts(settings.layout, layout))
 
     def show_png(self):
-        """Display the widget in PNG format."""
+        """Display the figure in PNG format."""
         self.show(renderer="png", width=self.layout.width, height=self.layout.height)
 
 
-class CustomFigureWidget(go.FigureWidget):
+class FigureWidget(go.FigureWidget):
     def __init__(self, *args, **kwargs):
         """Subclass of the `plotly.graph_objects.FigureWidget` class initialized
         with default parameters from `vectorbt.settings.layout`."""
@@ -28,10 +29,13 @@ class CustomFigureWidget(go.FigureWidget):
 
         layout = kwargs.pop('layout', {})
         super().__init__(*args, **kwargs)
-        self.update_layout(**settings.layout)
-        self.update_layout(colorway=list(settings.color_schema.values()))
-        self.update_layout(**layout)
+        self.update_layout(**merge_dicts(settings.layout, layout))
 
     def show_png(self):
         """Display the widget in PNG format."""
         self.show(renderer="png", width=self.layout.width, height=self.layout.height)
+
+
+def make_subplots(*args, **kwargs):
+    """Makes subplots and passes them to `FigureWidget`."""
+    return FigureWidget(_make_subplots(*args, **kwargs))
